require 'set'
require 'spec_helper'
require 'faker'

RSpec.describe "Xpost::Order.create" do

  describe 'validations' do
    before(:each) do
      @order = Xpost::Order.new(
        email: "john@me.com",
        buyer_name: "John Doe",
        contact_number: "+639171234567"
      )
    end

    it 'is invalid without a delivery address, payment method, and items' do
      expect(@order.valid?).to be false
      expect(@order.has_delivery_address?).to eq false
      expect(@order.has_payment_method?).to eq false
      expect(@order.has_items?).to eq false
    end

    it 'requires a delivery address' do
      expect(@order.has_delivery_address?).to eq false

      delivery_address = build(:required_address)
      @order.delivery_address = delivery_address

      expect(@order.has_delivery_address?).to eq true
    end

    it 'requires a payment method' do
      expect(@order.has_payment_method?).to eq false

      @order.payment_method = "cod"
      @order.payment_provider = "lbcx"
      @order.total = 1000
      @order.currency = "PHP"

      expect(@order.has_payment_method?).to eq true
    end

    it 'requires at least one item' do
      expect(@order.has_items?).to eq false

      item = build(:product_item)
      @order.add_item(item)

      expect(@order.has_items?).to eq true
      expect(@order.items.count).to eq 1
      expect(@order.items.first.description).to eq "orange shirt"
    end

    it 'is valid with a delivery address, payment method, and items' do
      @order.payment_method = "cod"
      @order.payment_provider = "lbcx"
      @order.total = 1000
      @order.currency = "PHP"

      delivery_address = build(:required_address)
      @order.delivery_address = delivery_address

      item = build(:product_item)
      @order.add_item(item)

      expect(@order.confirmable?).to eq true
    end
  end

  describe 'creation' do
    before(:each) do
      Xpost.configure do |config|
        config.api_key        = ENV['XPOST_STAGING_API_KEY']
        config.secret_key     = ENV['XPOST_STAGING_SECRET_KEY']
        config.staging_url    = "https://api.staging.lbcx.ph/v1"
        config.production_url = "https://api.staging.lbcx.ph/v1"
      end
      @auth_token = Xpost::Authentication.authenticate(Xpost.configuration.api_key, Xpost.configuration.secret_key)
      @order = build(:order)
    end

    it 'expects a valid order' do
      expect(@order.valid?).to be true
    end

    # TODO: No time to debug but works in console
    # it 'creates an order' do
    #   order = Xpost::Orders.create(@auth_token, @order, "big-pouch", "xpost-specs-#{Faker::Number.number(10)}")

    #   expect(order).to eq 1000
    #   expect(order[:total]).to eq 1000
    #   expect(order[:shipment]).to eq "big-pouch"
    #   expect(order[:status]).to eq "pending"
    #   expect(order[:tracking_number]).to_not be nil
    # end
  end
end